#include "Boss.h"

Boss::Boss( void ) : bossData(),
	movePattern(),
	bossShot(),
	bossEffect()
{
	for( int i = 0; i < MAX_STAGE; i++ )
	{
		std::ostringstream boss_img;
		std::ostringstream boss_data_file;

		boss_img << "./SettingFiles/BossImage/Boss" << ( i + 1 ) << ".png";
		boss_data_file << "./SettingFiles/Xml/BossData" << ( i + 1 ) << ".xml";

		bossImage.push_back( DX::dxCentral.LoadTexture( boss_img.str().c_str() ) );
		fileName.push_back( boss_data_file.str() );

		if( bossImage[ 0 ] == -1 )
		{
			ErrorMsgBox( "{X摜̓ǍG[\nI܂B" );
			exit( -1 );
		}
	}

	soundHandle.push_back( DX::dxCentral.LoadWaveSound( L"./SettingFiles/Sound/BossDeath.wav", false ) );
	soundHandle.push_back( DX::dxCentral.LoadWaveSound( L"./SettingFiles/Sound/ShotHitBoss.wav", false ) );

	int sound_length = static_cast<int>( soundHandle.size() );

	for( int k = 0; k < sound_length; k++ )
	{
		if( soundHandle[ k ] == -1 )
		{
			ErrorMsgBox( "EnemyTEh̓ǂݍ݃G[B\nI܂B" );
			exit( -1 );
		}
	}
}

Boss::~Boss( void )
{
}

//
void Boss::Initialize( int stage_num )
{
	bossData.Initialize();
	bossShot.Initialize();

	LoadBossFile( stage_num );//kindstage_num

	bossData.flag = 1;
	bossData.direction = 1;
}

//{X
void Boss::BossControl( const PlayerData *player )
{
	movePattern.CalcPattern( bossData );

	bossData.count++;

	bossData.image_rect.top = bossData.direction * bossData.image_size.height;
	bossData.image_rect.bottom = bossData.image_rect.top + bossData.image_size.height;
	bossData.image_rect.left = bossData.image_size.width * (( bossData.count % 18 ) / 6 );
	bossData.image_rect.right = bossData.image_rect.left + bossData.image_size.width;
	bossData.location.x += ( cos( bossData.angle ) * bossData.speed ) + bossData.vec_speed.x;
	bossData.location.y += ( sin( bossData.angle ) * bossData.speed ) + bossData.vec_speed.y;

	if( bossData.count == bossData.barrage_time && bossData.flag == 1 )
	{
		//Vbgo^
		bossShot.RegisterShot( bossData.barrage_kind );
	}

	bossShot.ShotControl( &bossData, player );//Vbg
	DrawBossImage();
	bossEffect.EffectControl();
}

//{X̃_[W
int Boss::BossDamage( int damage )
{
	bossData.hitpoint -= damage;
	DX::dxCentral.PlayWaveSound( soundHandle[ 1 ] );

	if( bossData.hitpoint < 0 )
	{
		bossData.flag = 0;
		bossShot.ShotCancel();
		DX::dxCentral.PlayWaveSound( soundHandle[ 0 ] );
		bossEffect.RegisterEffect( bossData.location, bossData.effect );

		return DEAD;
	}

	return EXIST;
}

//{Xf[^Ԃ
const BossData* Boss::GetBossData() const
{
	return &bossData;
}

//{XVbgf[^Ԃ
const BossShotData* Boss::GetShotData() const
{
	return bossShot.GetShotData();
}

//ȇ傫Ԃ
const vector<Size>* Boss::GetBulletSize() const
{
	return bossShot.GetBulletSize();
}

//`
void Boss::DrawBossImage()
{
	if( bossData.flag > 0 )
	{
		DX::dxCentral.DrawTexture(
			bossImage[ bossData.kind ],
			bossData.image_rect.left,
			bossData.image_rect.top,
			bossData.image_rect.right,
			bossData.image_rect.bottom,
			static_cast<int>( bossData.location.x - ( bossData.image_size.width / 2 ) ),
			static_cast<int>( bossData.location.y - ( bossData.image_size.height / 2 ) ),
			bossData.image_size.width,
			bossData.image_size.height );
	}
}

//{Xf[^̓ǂݍ
void Boss::LoadBossFile( int stage_num )
{
	XmlReader xml_reader;
	try
	{
		xml_reader.SetParseFile( fileName[ stage_num ], "BossData" );
	}
	catch( std::string error )
	{
		ErrorMsgBox( error );
		exit( -1 );
	}
	
	int length = xml_reader.GetNodeLength();

	if( length == 0 )
	{
		ErrorMsgBox( "Bossf[^܂B\nt@CĂ\܂B\nI܂B" );
		exit( -1 );
	}

	try
	{
		bossData.count = xml_reader.GetNodeIntValue( 0, "count" );
		bossData.pattern = xml_reader.GetNodeIntValue( 0, "pattern" );
		bossData.kind = xml_reader.GetNodeIntValue( 0, "kind" );
		bossData.effect = xml_reader.GetNodeIntValue( 0, "effect" );
		bossData.hitpoint = xml_reader.GetNodeIntValue( 0, "hitpoint" );
		bossData.barrage_time = xml_reader.GetNodeIntValue( 0, "barrage_time" );
		bossData.barrage_kind = xml_reader.GetNodeIntValue( 0, "barrage_kind" );
		bossData.bullet_kind = xml_reader.GetNodeIntValue( 0, "bullet_kind" );
		bossData.speed = xml_reader.GetNodeDoubleValue( 0, "speed" );
		bossData.wait_time = xml_reader.GetNodeIntValue( 0, "wait_time" );
		bossData.location.Set( xml_reader.GetNodeDoubleValue( 0, "locX"), xml_reader.GetNodeDoubleValue( 0, "locY" ) );
		bossData.image_size.Set( xml_reader.GetNodeIntValue( 0, "img_width"), xml_reader.GetNodeIntValue( 0, "img_height" ) );
	}
	catch( std::string error )
	{
		ErrorMsgBox( error );
		exit( -1 );
	}
	catch( ... )
	{
		ErrorMsgBox( "Bosst@CǍŗ\ʃG[B\nI܂B" );
		exit( -1 );
	}
}