#include "DxAudio.h"

//RXgN^@o
DxAudio::DxAudio(void)
{
	xAudio = NULL;
	masteringVoice = NULL;

	for( int i = 0; i < MAX_SOUND; i++ )
	{
		sourceVoice[i] = NULL;
		xAudioBuffer[i] = NULL;
		waveData[i] = NULL;
	}
}

//fXgN^@o
DxAudio::~DxAudio(void)
{
	for( int i = 0; i < MAX_SOUND; i++ )
	{
		if( waveData[ i ] != NULL )
		{
			SAFE_DELETE( waveData[i] );
		}

		if( xAudioBuffer[ i ] != NULL )
		{
			SAFE_DELETE( xAudioBuffer[ i ] );
		}

		if( sourceVoice[ i ] != NULL )
		{
			sourceVoice[ i ]->DestroyVoice();
			sourceVoice[ i ] = NULL;
		}
	}

	if( masteringVoice != NULL )
	{
		masteringVoice->DestroyVoice();
		masteringVoice = NULL;
	}
	
	if( xAudio != NULL )
	{
		SAFE_RELEASE( xAudio );
	}

	CoUninitialize();
}

//XAudio2
int DxAudio::InitializeXAudio()
{
	if( FAILED( CoInitializeEx( NULL, COINIT_MULTITHREADED ) ) )
	{
		return -1;
	}

	if( FAILED( XAudio2Create( &xAudio, ( UINT32 )0 ) ) )
	{
		CoUninitialize();
		return -1;
	}

	if( FAILED( xAudio->CreateMasteringVoice( &masteringVoice ) ) )
	{
		SAFE_RELEASE( xAudio );
		CoUninitialize();
		return -1;
	}

	return 0;
}

//waveTEh̓ǂݍ݁@ANZXL[Ԃ errorȂ-1Ԃ
int DxAudio::LoadWaveSound( LPTSTR filename, bool loopflag )
{
	int empty_num = -1;
	CWaveFile wav;

	for( int i = 0; i < MAX_SOUND; i++ )
	{
		if( sourceVoice[i] == NULL )
		{
			empty_num = i;
			break;
		}
	}

	if( empty_num == -1 )
	{
		return -1;//s
	}

	if( FAILED( wav.Open( filename, NULL ,WAVEFILE_READ ) ) )
	{
		return -1;
	}

	if( FAILED( xAudio->CreateSourceVoice( &sourceVoice[ empty_num ], wav.GetFormat() ) ) )
	{
		return -1;
	}
	
	DWORD size = wav.GetSize();
	waveData[ empty_num ] = new BYTE[ size ];

	if( FAILED( wav.Read( waveData[ empty_num ], size, &size ) ) )
	{
		SAFE_DELETE_ARRAY( waveData[ empty_num ] );
		return -1;
	}
	
	xAudioBuffer[ empty_num ] = new XAUDIO2_BUFFER;
	xAudioBuffer[ empty_num ]->Flags = XAUDIO2_END_OF_STREAM;
	xAudioBuffer[ empty_num ]->AudioBytes = size;
	xAudioBuffer[ empty_num ]->pAudioData = waveData[ empty_num ];
	xAudioBuffer[ empty_num ]->PlayBegin = 0;
	xAudioBuffer[ empty_num ]->PlayLength = 0;
	xAudioBuffer[ empty_num ]->LoopBegin = 0;
	xAudioBuffer[ empty_num ]->LoopLength = 0;
	xAudioBuffer[ empty_num ]->LoopCount = 0;
	
	if( loopflag )//trueȂ烋[vݒ
	{
		xAudioBuffer[ empty_num ]->LoopCount = XAUDIO2_LOOP_INFINITE;
	}

	if( FAILED( sourceVoice[ empty_num ]->SubmitSourceBuffer( xAudioBuffer[ empty_num ] ) ) )
	{
		sourceVoice[ empty_num ]->DestroyVoice();
		SAFE_DELETE_ARRAY( waveData[ empty_num ] );
		return -1;
	}

	return empty_num;
}

//TEh̍Đ G[Ȃ-1@O
int DxAudio::PlayWaveSound( int index )
{
	if( index < 0 || index >= MAX_SOUND || sourceVoice[ index ] == NULL )
	{
		return -1;
	}

	sourceVoice[ index ]->Stop( 0 );
	sourceVoice[ index ]->FlushSourceBuffers();
	sourceVoice[ index ]->SubmitSourceBuffer( xAudioBuffer[ index ] );

	if( FAILED( sourceVoice[ index ]->Start( 0 ) ) )
	{
		return -1;
	}

	return 0;
}

//ʂݒ肷BG[-1
int DxAudio::SetSoundVolume( int index, float volume )
{
	if( index < 0 || index >= MAX_SOUND || sourceVoice[ index ] == NULL )
	{
		return -1;
	}

	//őQC 144.5dBB(QCƂ́AdCH̑ɂēdCM𑝕邱ƁA܂͂̑̒l̂ƂłBidB) )
	//1.0 ̃{[́A܂̓QCȂƂӖ܂B0 ̃{[͖Ӗ܂B
	if( FAILED( sourceVoice[ index ]->SetVolume( volume ) ) )
	{
		return -1;
	}

	return 0;
}

//TEh̍폜
void DxAudio::DeleteSound( int index )
{
	if( waveData[ index ] != NULL )
	{
		SAFE_DELETE( waveData[index] );
	}

	if( xAudioBuffer[ index ] != NULL )
	{
		SAFE_DELETE( xAudioBuffer[ index ] );
	}

	if( sourceVoice[ index ] != NULL )
	{
		sourceVoice[ index ]->Stop( 0 );
		sourceVoice[ index ]->DestroyVoice();
		sourceVoice[ index ] = NULL;
	}
}

void DxAudio::DeleteAllSound()
{
	for( int i = 0; i < MAX_SOUND; i++ )
	{
		if( waveData[ i ] != NULL )
		{
			SAFE_DELETE( waveData[i] );
		}

		if( xAudioBuffer[ i ] != NULL )
		{
			SAFE_DELETE( xAudioBuffer[ i ] );
		}

		if( sourceVoice[ i ] != NULL )
		{
			sourceVoice[ i ]->Stop( 0 );
			sourceVoice[ i ]->DestroyVoice();
			sourceVoice[ i ] = NULL;
		}
	}
}